#include <sys/types.h>
#include <sys/time.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <pcap.h>

static void my_pcap_dump(u_char *, const struct pcap_pkthdr *, const u_char *);

typedef struct {
	pcap_dumper_t *out;
	struct timeval	 last;
} my_pcap_data;

int
main(int argc, char *argv[])
{
	const char *infname, *outfname;
	pcap_t *in;
	my_pcap_data d;
	char errbuf[2048];

	if (argc > 1)
		infname = argv[1];
	else
		infname = "-";
	if (argc > 2)
		outfname = argv[2];
	else
		outfname = "-";

	*errbuf = '\0';
	in = pcap_open_offline(infname, errbuf);
	if (in == NULL) {
		fprintf(stderr, "Error opening input file %s: %s\n",
		    infname, errbuf);
		exit(1);
	} else if (errbuf[0] != '\0') {
		fprintf(stderr, "Warning opening input file %s: %s\n",
		    infname, errbuf);
	}
	d.out = pcap_dump_open(in, outfname);
	if (d.out == NULL) {
		fprintf(stderr, "Error opening output file %s: %s\n",
		    outfname, pcap_geterr(in));
		exit(1);
	}
	/* GACK! */
	setbuf((FILE *)d.out, NULL);

	memset(&d.last, 0, sizeof(d.last));
	pcap_loop(in, -1, my_pcap_dump, (u_char *)&d);

	pcap_dump_close(d.out);
	pcap_close(in);
	return (0);
}

void my_pcap_dump(u_char *data, const struct pcap_pkthdr *pkt,
		const u_char *pktdata)
{
	unsigned long diff;
	my_pcap_data *d = (my_pcap_data *)data;
	
	if (d->last.tv_sec != 0) {
		diff = (pkt->ts.tv_sec * 1000000 + pkt->ts.tv_usec) -
		    (d->last.tv_sec * 1000000 + d->last.tv_usec);
		if (diff < 2000) {
			/*
			fprintf(stderr, "Ignoring a small diff %lu\n", diff);
			*/
		} else {
			usleep(diff);
		}
	}
	memcpy(&d->last, &pkt->ts, sizeof(d->last));
	/*
	fprintf(stderr, "Dumping a packet of %lu/%lu\n",
	    (unsigned long)pkt->len, (unsigned long)pkt->caplen);
	    */
	pcap_dump((u_char *)d->out, pkt, pktdata);
}
